library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;


entity MAC_DFSM is
    Port ( b : in std_logic;   -- enable signal that tells the module to transition in the FSM
			  i : in integer range 0 to 255;	-- Input values
			  clk, rst : in std_logic;
			  done, valid : out std_logic;   -- if there is atleast one valid result valid and done should be high.
			  cnt : out integer range 0 to 255; -- Need a counter to keep track of results since we can use several substrings.
           acc : out  integer range 0 to 65280); -- Result vector
end MAC_DFSM;

architecture rtl of MAC_DFSM is
	type int_array is array(0 to 2) of integer range 0 to 255;
	signal reg_i: int_array; 
	signal reg_o: integer range 0 to 65280;
	
	signal substr_cnt: integer range 0 to 255; -- could be useful I guess?

	type states is ( s0, s1 , s2, s3);
	
	signal current_state, next_state : states;
begin
	state_reg: process (rst, clk)
	begin
		if (rising_edge(clk)) then
			if(rst='1') then
				current_state <= s0;
			else
				current_state <= next_state;
			end if;
		end if;
	end process;
	
	next_state_logic: process(current_state, b, substr_cnt)
	begin
		case current_state is
		when s0 => -- initial state, if valid input occur jump to s1
			if (b = '1') then
				next_state <= s1;
			else
				next_state <= s0;
			end if;
		when s1 => -- processing state. Will keep adding and multiplying until it receives a b='0'
			if ( b = '1') then
				next_state <= s1;
			elsif ( b = '0' and substr_cnt > 0 ) then
				next_state <= s2; 
			else 
				next_state <= s3;
			end if;
		when s2 =>
			next_state <= s0; -- some valid inputs but the valid inputs stopped coming.
		when s3 =>
			next_state <= s0; -- invalid input! This clone will be deleted in a NFSM.
		end case;
	end process;
	
	output_logic: process(current_state, i, reg_o, substr_cnt)
	begin
		case current_state is
			when s0 =>
				done <= '0';
				valid <= '0';
				acc <= 0;
				cnt <= 0;
			when s1 =>
				done <= '0';
				valid <= '1';
				acc <= reg_o;
				cnt <= substr_cnt;
			when s2 =>
				done <= '1';
				valid <= '1';
				acc <= reg_o;
				cnt <= substr_cnt;
			when s3 =>
				done <= '1';
				valid <= '0';
				acc <= 0;
				cnt <= 0;				
		end case;
	end process;
	
	load_reg: process(clk) -- loading the input shift register
	begin
		if (rising_edge(clk)) then
			reg_i(2) <= reg_i(1);
			reg_i(1) <= reg_i(0);
			reg_i(0) <= i;
		end if;
	end process;
	
	prep_output: process(clk)
	begin
		if (rising_edge(clk)) then
			case current_state is
				when s0 =>
					reg_o <= 0;
					substr_cnt <= 0;
				when s1 =>
					reg_o <= reg_i(0) + (reg_i(2) * reg_i(1));
					substr_cnt <= substr_cnt + 1;
				when s2 =>
					reg_o <= reg_o;
					substr_cnt <= substr_cnt;
				when s3 =>
					reg_o <= reg_o;
					substr_cnt <= substr_cnt;
			end case;
		end if;
	end process;
	
end rtl;

