library IEEE;
use IEEE.STD_LOGIC_1164.ALL;

entity control is
	port ( clk, reset, transition_sig: in std_logic;
			 done_vec: in std_logic_vector(3 downto 0);
			 en_vec: out std_logic_vector(3 downto 0)
	);
end control;

architecture Behavioral of control is
type control_state_type is (init, active);

signal current_state: control_state_type;
signal next_state: control_state_type;

signal done_reg: std_logic_vector(3 downto 0);

signal en_reg: std_logic_vector(3 downto 0);
begin
	state_reg: process(clk)
	begin
		if(rising_edge(clk)) then
			if(reset = '1') then
				current_state <= init;
			else
				current_state <= next_state;
			end if;
		end if;
	end process;
	
	next_state_logic: process(current_state, transition_sig)
	begin
		case current_state is
			when init =>
				if( transition_sig = '1') then
					next_state <= active;
				else
					next_state <= init;
				end if;
			when active =>
				if( transition_sig = '1') then
					next_state <= active;
				else
					next_state <= init;
				end if;
		end case;		
	end process;
	
	reg_load: process(clk, done_vec, current_state, en_reg, transition_sig)
	begin
		if(rising_edge(clk)) then
			done_reg <= done_vec; -- not needed unless result management integrated in control
			
			case current_state is
				when init => 
					if( transition_sig /= '1') then
						en_reg <= "0001";
					else
						en_reg(3 downto 1) <= en_reg(2 downto 0);                               
						en_reg(0) <= '1';
					end if;
				when active =>
					if( transition_sig = '1') then
						en_reg(3 downto 1) <= en_reg(2 downto 0);                               
						en_reg(0) <= '1';
					else
						en_reg <= "0000";
					end if;
			end case;
		end if;                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                            
		en_vec <= en_reg;
	end process;
end Behavioral;

