library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.sing_stat_package.all;

entity data_path is
    Port ( clk : in std_logic;
			  state_vector: in std_logic_vector((FSM_SIZE-1) downto 0);
			  i : in integer range 0 to 255;
			  done, valid : out std_logic; 
           acc : out  integer range 0 to 65280
			  ); 
end data_path;

architecture Behavioral of data_path is
	signal reg_i: integer range 0 to 255;
	signal reg_o: integer range 0 to 65280;
	signal current_state: std_logic_vector((FSM_SIZE-1) downto 0);
	
begin
	
	current_state <= state_vector;
	 
	output_logic: process(current_state, reg_o)
	begin
		case current_state is
			when "0001" =>
				done <= '0';
				valid <= '0';
				acc <= reg_o;
			when "0010" =>
				done <= '0';
				valid <= '1';
				acc <= reg_o;
			when "0100" =>
				done <= '0';
				valid <= '1';
				acc <= reg_o;
			when "1000" =>
				done <= '1';
				valid <= '1';
				acc <= reg_o;
			when others =>
				done <= '0';
				valid <= '0';
				acc <= 0;
		end case;
	end process;

	load_reg: process(clk)
	begin
		if (rising_edge(clk)) then
			reg_i <= i;
		end if;
	end process;
	
	prep_output: process(clk)
	begin
		if (rising_edge(clk)) then
			case current_state is
				when "0001" =>
					reg_o <= reg_i;
				when "0010" =>
					reg_o <= reg_o * reg_i;
				when "0100" =>
					reg_o <= reg_o + reg_i;
				when "1000" =>
					reg_o <= reg_o;
				when others =>
					reg_o <= 0;
			end case;
		end if;
	end process;

end Behavioral;

