library IEEE;
use IEEE.STD_LOGIC_1164.all;

package sing_stat_package is
	constant FSM_SIZE : integer := 4; -- Size of FSM
	constant EXT_INPUT_NUM : integer := 1; -- Number of external inputs
	constant EN_NUM: integer := FSM_SIZE + EXT_INPUT_NUM; -- number of enable signals in the singular state
	constant CONF_DATA_LEN: integer := 5; -- Size of config data register in each singular state.
	
   type state_type is (init, read_config, active, not_active);

	subtype conf_info is std_logic_vector(EN_NUM - 1 downto 0);		
	type conf_data is array(integer range 0 to CONF_DATA_LEN-1) of conf_info;
	type conf_vecs is array(integer range 0 to FSM_SIZE - 1) of conf_info;

function check_config(signal valid : conf_data; signal input : conf_info) return std_logic;
	
end sing_stat_package;

package body sing_stat_package is
	function check_config(signal valid : conf_data; signal input: conf_info) return std_logic is
		variable hit: std_logic;
		begin
			hit := '0';
			for i in 0 to (CONF_DATA_LEN - 1) loop
				if (valid(i) = input ) then
					hit := '1';
				end if;
			end loop;
			return hit;
	end check_config;
 
end sing_stat_package;
