------------ singular state -----------------
library ieee;
use ieee.std_logic_1164.all;
use work.sing_stat_package.all;
use work.reduce_pack.all;
--------------------------------------------
entity singular_state is
	port (input, reset, clk, fsm_en: in std_logic;
			en0, en1, en2, en3: in std_logic;
			conf_en: in std_logic;
			conf_bits: in conf_info;
			ext_enable: out std_logic
			);
end singular_state;
--------------------------------------------

architecture arch of singular_state is
   signal current_state   : state_type;
	signal next_state : state_type;
	
	signal conf_reg: conf_data;
	signal enable_vec: conf_info;
	signal count_done: std_logic;
	
	signal initial_state: std_logic;
	
begin

	state_reg: process(clk)
	begin
		if(rising_edge(clk)) then
			if(reset = '1') then
				current_state <= init;
			else
				current_state <= next_state;
			end if;
		end if;
	end process;

	enable_vec <= input & en0 & en1 & en2 & en3;
	
	next_state_logic: process(current_state, conf_en, conf_reg, initial_state, enable_vec, count_done, fsm_en)
	begin
		case current_state is
			when init =>
				if( conf_en = '1' ) then
					next_state <= read_config;
				else
					next_state <= init;
				end if;
			when read_config =>
				if( count_done = '0' ) then
					next_state <= read_config;
				else
					if( initial_state = '1' ) then
						next_state <= active;	
					else
						next_state <= not_active;	
					end if;
				end if;
			when active =>
					if(check_config(conf_reg, enable_vec) = '1' and fsm_en = '1') then
						next_state <= active;
					elsif( initial_state = '1' and fsm_en = '0') then
						next_state <= active;	
					else
						next_state <= not_active;	
					end if;
			when not_active =>
					if(check_config(conf_reg, enable_vec) = '1' and fsm_en = '1') then
						next_state <= active;
					elsif( initial_state = '1' and fsm_en = '0') then
							next_state <= active;	
					else
							next_state <= not_active;	
					end if;
		end case;
	end process;
	
	load_regs: process(clk)
	variable count: integer range 0 to CONF_DATA_LEN-1;
	begin
		if (rising_edge(clk)) then -- here is the error. Will not reach read_config until the very_top_level FSM is enabled in the system.
			case current_state is
				when init =>
					conf_reg <=  (others => (others => '0'));
					count := 0;
					count_done <= '0';
				when read_config =>
					conf_reg(1 to CONF_DATA_LEN-1) <= conf_reg(0 to CONF_DATA_LEN-2);
					conf_reg(0) <= conf_bits;
					count := count + 1;
					if (count = CONF_DATA_LEN - 1) then 
						count_done <= '1';
					end if;
					
					if(or_reduce(conf_bits) = '0') then
						initial_state <= '1';
					end if;
				when active =>
					-- Do nothing
				when not_active =>
					-- Do nothing
			end case;
		end if;
	end process;

	output_logic : process(current_state)
	begin
		case current_state is
			when init =>
				ext_enable <= '0';
			when read_config =>
				ext_enable <= '0';
			when active =>
				ext_enable <= '1';
			when not_active =>
				ext_enable <= '0';
		end case;
	end process;
end arch;