library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use work.sing_stat_package.all;

entity system is
	port( clk, reset, transition_sig, conf_en: in std_logic;
			conf_vecs: in conf_vecs;
			input_data: in integer range 0 to 255;
			result_out: out integer range 0 to 65280
		);
end system;

architecture Behavioral of system is

component control
	port ( clk, reset, transition_sig: in std_logic;
			 done_vec: in std_logic_vector(3 downto 0);
			 en_vec: out std_logic_vector(3 downto 0)
	);
end component;

component very_top_level
	port ( transition_sig, conf_en, reset, clk, fsm_en: in std_logic;
			 conf_vec_0, conf_vec_1, conf_vec_2, conf_vec_3: in conf_info;
			 input_data: in integer range 0 to 255;
			 done, valid : out std_logic;
			 result: out integer range 0 to 65280
	);
end component;

signal en_vec: std_logic_vector(3 downto 0);
signal done_vec: std_logic_vector(3 downto 0);
signal valid_vec: std_logic_vector(3 downto 0);
	
type results_data is array(integer range 0 to 3) of integer range 0 to 65280;
signal results_out_reg: results_data; -- register to send out the correct result every cycle.
signal results_temp_reg: results_data; -- register to receive the result from the different FSMs

begin
	c0: control port map (
		transition_sig=>transition_sig, 
		reset=>reset, 
		clk => clk,
		done_vec => done_vec,
		en_vec => en_vec
		);
		
	v0: very_top_level port map (
		clk => clk,
		reset => reset,
		transition_sig => transition_sig,
		fsm_en => en_vec(0),
		valid => valid_vec(0),
		done => done_vec(0),
		conf_en => conf_en,
		conf_vec_0 => conf_vecs(0),
		conf_vec_1 => conf_vecs(1),
		conf_vec_2 => conf_vecs(2),
		conf_vec_3 => conf_vecs(3),
		input_data => input_data,
		result => results_temp_reg(0)
		);
		
	v1: very_top_level port map (
		clk => clk,
		reset => reset,
		transition_sig => transition_sig,
		fsm_en => en_vec(1), -- problem here since the states won't be configured if it cannont transition between internal states. Need to change the internal logic to let it be able to configure itself.
		valid => valid_vec(1),
		done => done_vec(1),
		conf_en => conf_en,
		conf_vec_0 => conf_vecs(0),
		conf_vec_1 => conf_vecs(1),
		conf_vec_2 => conf_vecs(2), 
		conf_vec_3 => conf_vecs(3),
		input_data => input_data,
		result => results_temp_reg(1)
		);
		
	v2: very_top_level port map (
		clk => clk,
		reset => reset,
		transition_sig => transition_sig,
		fsm_en => en_vec(2),
		valid => valid_vec(2),
		done => done_vec(2),
		conf_en => conf_en,
		conf_vec_0 => conf_vecs(0),
		conf_vec_1 => conf_vecs(1),
		conf_vec_2 => conf_vecs(2),
		conf_vec_3 => conf_vecs(3),
		input_data => input_data,
		result => results_temp_reg(2)
		);
		
	v3: very_top_level port map (
		clk => clk,
		reset => reset,
		transition_sig => transition_sig,
		fsm_en => en_vec(3),
		valid => valid_vec(3),
		done => done_vec(3),
		conf_en => conf_en,
		conf_vec_0 => conf_vecs(0),
		conf_vec_1 => conf_vecs(1),
		conf_vec_2 => conf_vecs(2),
		conf_vec_3 => conf_vecs(3),
		input_data => input_data,
		result => results_temp_reg(3)
		);
		
		result_sreg: process(clk)
		begin
			if(rising_edge(clk)) then
				if(reset = '1') then
					results_out_reg <= (others=>0);
				else
				
					result_out <= results_out_reg(3);
					results_out_reg(1 to 3) <= results_out_reg(0 to 2);
					
					if( done_vec = "0000" ) then
							results_out_reg(0) <= 0;
					else
						for i in 0 to 3 loop
							if(done_vec(i) = '1') then
								results_out_reg(0) <= results_temp_reg(i);
							end if;
						end loop;
					end if;
				end if;
			end if;
		end process;
end Behavioral;

