LIBRARY ieee;
USE ieee.std_logic_1164.ALL;
use work.sing_stat_package.all;
use work.reduce_pack.all;
 
ENTITY system_tb IS
END system_tb;
 
ARCHITECTURE behavior OF system_tb IS 
 
    COMPONENT system
    PORT(
         clk : IN  std_logic;
         reset : IN  std_logic;
         transition_sig : IN  std_logic;
         conf_en : IN  std_logic;
         conf_vecs : IN  conf_vecs;
         input_data : IN  integer range 0 to 255;
         result_out : OUT  integer range 0 to 65280
        );
    END COMPONENT;
    

   --Inputs
   signal clk : std_logic := '0';
   signal reset : std_logic := '0';
   signal transition_sig : std_logic := '0';
   signal conf_en : std_logic := '0';
   signal conf_vecs : conf_vecs := (others=>(others => '0'));
   signal input_data : integer range 0 to 255 := 0;

 	--Outputs
   signal result_out : integer range 0 to 65280;

   -- Clock period definitions
   constant clk_period : time := 10 ns;
 
BEGIN
 
	-- Instantiate the Unit Under Test (UUT)
   uut: system PORT MAP (
          clk => clk,
          reset => reset,
          transition_sig => transition_sig,
          conf_en => conf_en,
          conf_vecs => conf_vecs,
          input_data => input_data,
          result_out => result_out
        );

   -- Clock process definitions
   clk_process :process
   begin
		clk <= '0';
		wait for clk_period/2;
		clk <= '1';
		wait for clk_period/2;
   end process;
 

   -- Stimulus process
stim_proc: process
   begin		
      -- hold reset state for 100 ns.
      wait for 100 ns;	
		
      wait for clk_period*10;
      -- insert stimulus here 
		reset <= '1';
		wait for clk_period;
		reset <= '0';

		-- CONFIGURATE STATES 
		-- Fill the remaining slots in the config registers with the same value. If not they will oscillate between all being enabled and all being disabled since they will all think they are initial states.
		wait for clk_period;
		conf_en <= '1';
		wait for clk_period;
		conf_en <= '0';
		conf_vecs(0) <= "00000"; -- input - en0 - en1 - en2 - en3
		conf_vecs(1) <= "11000";
		conf_vecs(2) <= "10100";
		conf_vecs(3) <= "10010";
		wait for clk_period;
		conf_vecs(0) <= "10000"; -- input - en0 - en1 - en2 - en3
		conf_vecs(1) <= "11000";
		conf_vecs(2) <= "10100";
		conf_vecs(3) <= "10010";
		wait for clk_period;
		conf_vecs(0) <= "01000"; -- input - en0 - en1 - en2 - en3
		wait for clk_period;
		conf_vecs(0) <= "00001";
		wait for clk_period;
		conf_vecs(0) <= "10001";
		
		-- RUN STATE MACHINE
		
		wait for clk_period;
		transition_sig <= '1';
		for i in 1 to 200 loop
			input_data <= i;
			if( i = 8 ) then
				transition_sig <= '0';
			elsif (i = 19) then
				transition_sig <= '1';
			elsif (i = 13) then
				transition_sig <= '0';
			elsif (i = 14) then
				transition_sig <= '1';
			elsif (i = 15) then
				transition_sig <= '0';
			elsif (i = 16) then
				transition_sig <= '1';
			end if;
			wait for clk_period;
		end loop;
		wait for clk_period*2;
      wait;
   end process;
	
	monitor: process(clk)
	begin
		if(rising_edge(clk)) then
			assert result_out /= 0 report "Result_out equals zero! Check inputs and beware of errors." severity warning;
		end if;
	end process;

END;
